# Globals
def init():
    global VERSION
    VERSION = "0.8"
    global APP_NAME
    APP_NAME = "Raincoat"
    global TORRENTS
    TORRENTS = []
    global APIKEY
    APIKEY = ""
    global JACKETT_URL
    JACKETT_URL = ""
    global JACKETT_INDEXER
    JACKETT_INDEXER = ""
    global DESC_LENGTH
    DESC_LENGTH = ""
    global EXCLUDE
    EXCLUDE = ""
    global RESULTS_LIMIT
    RESULTS_LIMIT = ""
    global CLIENT_URL
    CLIENT_URL = ""
    global DISPLAY
    DISPLAY = ""
    global TOR_CLIENT
    TOR_CLIENT = ""
    global TOR_CLIENT_USER
    TOR_CLIENT_USER = ""
    global TOR_CLIENT_PW
    TOR_CLIENT_PW = ""
    global DOWNLOAD_DIR
    DOWNLOAD_DIR = ""
    global  CURRENT_PAGE
    CURRENT_PAGE = 0
    global VERBOSE_MODE
    VERBOSE_MODE = False




